@@ -22,22 +22,16 @@ SHOULD include dedicated exception logging.
 
 """
 
-import sys
-
 from oslo_log import log as logging
-import six
+
 import webob.exc
 from webob import util as woutil
 
-import nova.conf
 from nova.i18n import _, _LE
 
 LOG = logging.getLogger(__name__)
 
 
-CONF = nova.conf.CONF
-
-
 class ConvertedException(webob.exc.WSGIHTTPException):
     def __init__(self, code, title="", explanation=""):
         self.code = code
@@ -88,27 +82,31 @@ class NovaException(Exception):
                 message = self.msg_fmt % kwargs
 
             except Exception:
-                exc_info = sys.exc_info()
-                # kwargs doesn't match a variable in the message
-                # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation'))
-                for name, value in six.iteritems(kwargs):
-                    LOG.error("%s: %s" % (name, value))  # noqa
-
-                if CONF.fatal_exception_format_errors:
-                    six.reraise(*exc_info)
-                else:
-                    # at least get the core message out if something happened
-                    message = self.msg_fmt
+                # NOTE(melwitt): This is done in a separate method so it can be
+                # monkey-patched during testing to make it a hard failure.
+                self._log_exception()
+                message = self.msg_fmt
 
         self.message = message
         super(NovaException, self).__init__(message)
 
+    def _log_exception(self):
+        # kwargs doesn't match a variable in the message
+        # log the issue and the kwargs
+        LOG.exception(_LE('Exception in string format operation'))
+        for name, value in self.kwargs.items():
+            LOG.error("%s: %s" % (name, value))  # noqa
+
     def format_message(self):
         # NOTE(mrodden): use the first argument to the python Exception object
         # which should be our full NovaException message, (see __init__)
         return self.args[0]
 
+    def __repr__(self):
+        dict_repr = self.__dict__
+        dict_repr['class'] = self.__class__.__name__
+        return str(dict_repr)
+
 
 class EncryptionFailure(NovaException):
     msg_fmt = _("Failed to encrypt text: %(reason)s")
@@ -148,6 +146,17 @@ class CinderConnectionFailed(NovaException):
     msg_fmt = _("Connection to cinder host failed: %(reason)s")
 
 
+class UnsupportedCinderAPIVersion(NovaException):
+    msg_fmt = _('Nova does not support Cinder API version %(version)s')
+
+
+class CinderAPIVersionNotAvailable(NovaException):
+    """Used to indicate that a requested Cinder API version, generally a
+    microversion, is not available.
+    """
+    msg_fmt = _('Cinder API version %(version)s is not available.')
+
+
 class Forbidden(NovaException):
     msg_fmt = _("Forbidden")
     code = 403
@@ -161,10 +170,6 @@ class PolicyNotAuthorized(Forbidden):
     msg_fmt = _("Policy doesn't allow %(action)s to be performed.")
 
 
-class VolumeLimitExceeded(Forbidden):
-    msg_fmt = _("Volume resource quota exceeded")
-
-
 class ImageNotActive(NovaException):
     # NOTE(jruzicka): IncorrectState is used for volumes only in EC2,
     # but it still seems like the most appropriate option.
@@ -194,11 +199,6 @@ class InvalidBDMVolume(InvalidBDM):
                 "failed to get volume %(id)s.")
 
 
-class UnsupportedBDMVolumeAuthMethod(InvalidBDM):
-    msg_fmt = _("Block Device Mapping is Invalid: "
-                "%(auth_method)s is unsupported.")
-
-
 class InvalidBDMImage(InvalidBDM):
     msg_fmt = _("Block Device Mapping is Invalid: "
                 "failed to get image %(id)s.")
@@ -255,8 +255,32 @@ class VolumeAttachFailed(Invalid):
                 "Reason: %(reason)s")
 
 
-class VolumeUnattached(Invalid):
-    msg_fmt = _("Volume %(volume_id)s is not attached to anything")
+class MultiattachNotSupportedByVirtDriver(NovaException):
+    # This exception indicates the compute hosting the instance does not
+    # support multiattach volumes. This should generally be considered a
+    # 409 HTTPConflict error in the API since we expect all virt drivers to
+    # eventually support multiattach volumes.
+    msg_fmt = _("Volume %(volume_id)s has 'multiattach' set, "
+                "which is not supported for this instance.")
+    code = 409
+
+
+class MultiattachSupportNotYetAvailable(NovaException):
+    # This exception indicates the deployment is not yet new enough to support
+    # multiattach volumes, so a 409 HTTPConflict response is generally used
+    # for handling this in the API.
+    msg_fmt = _("Multiattach volume support is not yet available.")
+    code = 409
+
+
+class MultiattachNotSupportedOldMicroversion(Invalid):
+    msg_fmt = _('Multiattach volumes are only supported starting with '
+                'compute API version 2.60.')
+
+
+class MultiattachToShelvedNotSupported(Invalid):
+    msg_fmt = _("Attaching multiattach volumes is not supported for "
+                "shelved-offloaded instances.")
 
 
 class VolumeNotCreated(NovaException):
@@ -265,11 +289,34 @@ class VolumeNotCreated(NovaException):
                 " attempts. And its status is %(volume_status)s.")
 
 
+class ExtendVolumeNotSupported(Invalid):
+    msg_fmt = _("Volume size extension is not supported by the hypervisor.")
+
+
 class VolumeEncryptionNotSupported(Invalid):
     msg_fmt = _("Volume encryption is not supported for %(volume_type)s "
                 "volume %(volume_id)s")
 
 
+class TaggedAttachmentNotSupported(Invalid):
+    msg_fmt = _("Tagged device attachment is not yet available.")
+
+
+class VolumeTaggedAttachNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged volume attachment is not supported for this server "
+                "instance.")
+
+
+class VolumeTaggedAttachToShelvedNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged volume attachment is not supported for "
+                "shelved-offloaded instances.")
+
+
+class NetworkInterfaceTaggedAttachNotSupported(TaggedAttachmentNotSupported):
+    msg_fmt = _("Tagged network interface attachment is not supported for "
+                "this server instance.")
+
+
 class InvalidKeypair(Invalid):
     msg_fmt = _("Keypair data is invalid: %(reason)s")
 
@@ -332,7 +379,7 @@ class ApiVersionsIntersect(Invalid):
 # Cannot be templated as the error syntax varies.
 # msg needs to be constructed when raised.
 class InvalidParameterValue(Invalid):
-    msg_fmt = _("%(err)s")
+    msg_fmt = "%(err)s"
 
 
 class InvalidAggregateAction(Invalid):
@@ -360,10 +407,6 @@ class InvalidAggregateActionUpdateMeta(InvalidAggregateAction):
                 "%(aggregate_id)s. Reason: %(reason)s.")
 
 
-class InvalidGroup(Invalid):
-    msg_fmt = _("Group not valid. Reason: %(reason)s")
-
-
 class InvalidSortKey(Invalid):
     msg_fmt = _("Sort key supplied was not valid.")
 
@@ -443,6 +486,10 @@ class ServiceUnavailable(Invalid):
     msg_fmt = _("Service is unavailable at this time.")
 
 
+class ServiceNotUnique(Invalid):
+    msg_fmt = _("More than one possible service found.")
+
+
 class ComputeResourcesUnavailable(ServiceUnavailable):
     msg_fmt = _("Insufficient compute resources: %(reason)s.")
 
@@ -498,10 +545,6 @@ class DevicePathInUse(Invalid):
     code = 409
 
 
-class DeviceIsBusy(Invalid):
-    msg_fmt = _("The supplied device (%(device)s) is busy.")
-
-
 class InvalidCPUInfo(Invalid):
     msg_fmt = _("Unacceptable CPU info: %(reason)s")
 
@@ -579,6 +622,10 @@ class AgentBuildExists(NovaException):
                 "architecture %(architecture)s exists.")
 
 
+class VolumeAttachmentNotFound(NotFound):
+    msg_fmt = _("Volume attachment %(attachment_id)s could not be found.")
+
+
 class VolumeNotFound(NotFound):
     msg_fmt = _("Volume %(volume_id)s could not be found.")
 
@@ -638,6 +685,10 @@ class ImageNotFound(NotFound):
     msg_fmt = _("Image %(image_id)s could not be found.")
 
 
+class ImageDeleteConflict(NovaException):
+    msg_fmt = _("Conflict deleting image. Reason: %(reason)s.")
+
+
 class PreserveEphemeralNotSupported(Invalid):
     msg_fmt = _("The current driver does not support "
                 "preserving ephemeral partitions.")
@@ -655,10 +706,6 @@ class InstanceMappingNotFound(NotFound):
     msg_fmt = _("Instance %(uuid)s has no mapping to a cell.")
 
 
-class NetworkDuplicated(Invalid):
-    msg_fmt = _("Network %(network_id)s is duplicated.")
-
-
 class NetworkDhcpReleaseFailed(NovaException):
     msg_fmt = _("Failed to release IP %(address)s with MAC %(mac_address)s")
 
@@ -958,6 +1005,11 @@ class ServiceNotFound(NotFound):
     msg_fmt = _("Service %(service_id)s could not be found.")
 
 
+class ConfGroupForServiceTypeNotFound(ServiceNotFound):
+    msg_fmt = _("No conf group name could be found for service type "
+                "%(stype)s.")
+
+
 class ServiceBinaryExists(NovaException):
     msg_fmt = _("Service with host %(host)s binary %(binary)s exists.")
 
@@ -974,11 +1026,6 @@ class ComputeHostNotFound(HostNotFound):
     msg_fmt = _("Compute host %(host)s could not be found.")
 
 
-class ComputeHostNotCreated(HostNotFound):
-    msg_fmt = _("Compute host %(name)s needs to be created first"
-                " before updating.")
-
-
 class HostBinaryNotFound(NotFound):
     msg_fmt = _("Could not find binary %(binary)s on host %(host)s.")
 
@@ -1022,6 +1069,10 @@ class QuotaClassNotFound(QuotaNotFound):
     msg_fmt = _("Quota class %(class_name)s could not be found.")
 
 
+class QuotaClassExists(NovaException):
+    msg_fmt = _("Quota class %(class_name)s exists for resource %(resource)s")
+
+
 class QuotaUsageNotFound(QuotaNotFound):
     msg_fmt = _("Quota usage for project %(project_id)s could not be found.")
 
@@ -1077,10 +1128,6 @@ class SecurityGroupCannotBeApplied(Invalid):
                 " in order to apply security groups.")
 
 
-class SecurityGroupRuleExists(Invalid):
-    msg_fmt = _("Rule already exists in group: %(rule)s")
-
-
 class NoUniqueMatch(NovaException):
     msg_fmt = _("No Unique Match Found.")
     code = 409
@@ -1115,10 +1162,6 @@ class ConsoleLogOutputException(NovaException):
                 "%(instance_id)s. Reason: %(reason)s")
 
 
-class ConsolePoolNotFound(NotFound):
-    msg_fmt = _("Console pool %(pool_id)s could not be found.")
-
-
 class ConsolePoolExists(NovaException):
     msg_fmt = _("Console pool with host %(host)s, console_type "
                 "%(console_type)s and compute_host %(compute_host)s "
@@ -1250,10 +1293,6 @@ class InstanceTagNotFound(NotFound):
     msg_fmt = _("Instance %(instance_id)s has no tag '%(tag)s'")
 
 
-class RotationRequiredForBackup(NovaException):
-    msg_fmt = _("Rotation param is required for backup image_type")
-
-
 class KeyPairExists(NovaException):
     msg_fmt = _("Key pair '%(key_name)s' already exists.")
 
@@ -1401,11 +1440,11 @@ class OnsetFileLimitExceeded(QuotaError):
 
 
 class OnsetFilePathLimitExceeded(OnsetFileLimitExceeded):
-    msg_fmt = _("Personality file path too long")
+    msg_fmt = _("Personality file path exceeds maximum %(allowed)s")
 
 
 class OnsetFileContentLimitExceeded(OnsetFileLimitExceeded):
-    msg_fmt = _("Personality file content too long")
+    msg_fmt = _("Personality file content exceeds maximum %(allowed)s")
 
 
 class KeypairLimitExceeded(QuotaError):
@@ -1461,22 +1500,10 @@ class InstanceInfoCacheNotFound(NotFound):
                 "found.")
 
 
-class InvalidAssociation(NotFound):
-    msg_fmt = _("Invalid association.")
-
-
 class MarkerNotFound(NotFound):
     msg_fmt = _("Marker %(marker)s could not be found.")
 
 
-class InvalidInstanceIDMalformed(Invalid):
-    msg_fmt = _("Invalid id: %(instance_id)s (expecting \"i-...\")")
-
-
-class InvalidVolumeIDMalformed(Invalid):
-    msg_fmt = _("Invalid id: %(volume_id)s (expecting \"i-...\")")
-
-
 class CouldNotFetchImage(NovaException):
     msg_fmt = _("Could not fetch image %(image_id)s")
 
@@ -1535,12 +1562,6 @@ class InterfaceDetachFailed(Invalid):
                 "%(instance_uuid)s")
 
 
-class InstanceUserDataTooLarge(NovaException):
-    msg_fmt = _("User data too large. User data must be no larger than "
-                "%(maxsize)s bytes once base64 encoded. Your data is "
-                "%(length)d bytes")
-
-
 class InstanceUserDataMalformed(NovaException):
     msg_fmt = _("User data needs to be valid base 64.")
 
@@ -1631,10 +1652,6 @@ class ObjectActionError(NovaException):
     msg_fmt = _('Object action %(action)s failed because: %(reason)s')
 
 
-class CoreAPIMissing(NovaException):
-    msg_fmt = _("Core API extensions are missing: %(missing_apis)s")
-
-
 class AgentError(NovaException):
     msg_fmt = _('Error during following call to agent: %(method)s')
 
@@ -1661,41 +1678,10 @@ class InstanceGroupMemberNotFound(NotFound):
                 "id %(instance_id)s.")
 
 
-class InstanceGroupPolicyNotFound(NotFound):
-    msg_fmt = _("Instance group %(group_uuid)s has no policy %(policy)s.")
-
-
 class InstanceGroupSaveException(NovaException):
     msg_fmt = _("%(field)s should not be part of the updates.")
 
 
-class PluginRetriesExceeded(NovaException):
-    msg_fmt = _("Number of retries to plugin (%(num_retries)d) exceeded.")
-
-
-class ImageDownloadModuleError(NovaException):
-    msg_fmt = _("There was an error with the download module %(module)s. "
-                "%(reason)s")
-
-
-class ImageDownloadModuleMetaDataError(ImageDownloadModuleError):
-    msg_fmt = _("The metadata for this location will not work with this "
-                "module %(module)s.  %(reason)s.")
-
-
-class ImageDownloadModuleNotImplementedError(ImageDownloadModuleError):
-    msg_fmt = _("The method %(method_name)s is not implemented.")
-
-
-class ImageDownloadModuleConfigurationError(ImageDownloadModuleError):
-    msg_fmt = _("The module %(module)s is misconfigured: %(reason)s.")
-
-
-class SignatureVerificationError(NovaException):
-    msg_fmt = _("Signature verification for the image "
-                "failed: %(reason)s.")
-
-
 class ResourceMonitorError(NovaException):
     msg_fmt = _("Error when creating resource monitor: %(monitor)s")
 
@@ -1704,11 +1690,6 @@ class PciDeviceWrongAddressFormat(NovaException):
     msg_fmt = _("The PCI address %(address)s has an incorrect format.")
 
 
-class PciDeviceInvalidAddressField(NovaException):
-    msg_fmt = _("Invalid PCI Whitelist: "
-                "The PCI address %(address)s has an invalid %(field)s.")
-
-
 class PciDeviceInvalidDeviceName(NovaException):
     msg_fmt = _("Invalid PCI Whitelist: "
                 "The PCI whitelist can specify devname or address,"
@@ -1767,17 +1748,16 @@ class PciRequestAliasNotDefined(NovaException):
     msg_fmt = _("PCI alias %(alias)s is not defined")
 
 
-class MissingParameter(NovaException):
-    msg_fmt = _("Not enough parameters: %(reason)s")
-    code = 400
-
-
 class PciConfigInvalidWhitelist(Invalid):
-    msg_fmt = _("Invalid PCI devices Whitelist config %(reason)s")
+    msg_fmt = _("Invalid PCI devices Whitelist config: %(reason)s")
 
 
 # Cannot be templated, msg needs to be constructed when raised.
 class InternalError(NovaException):
+    """Generic hypervisor errors.
+
+    Consider subclassing this to provide more specific exceptions.
+    """
     msg_fmt = "%(err)s"
 
 
@@ -1802,6 +1782,10 @@ class VolumesNotRemoved(Invalid):
     msg_fmt = _("Failed to remove volume(s): (%(reason)s)")
 
 
+class VolumeRebaseFailed(NovaException):
+    msg_fmt = _("Volume rebase failed: %(reason)s")
+
+
 class InvalidVideoMode(Invalid):
     msg_fmt = _("Provided video model (%(model)s) is not supported.")
 
@@ -1816,21 +1800,34 @@ class RequestedVRamTooHigh(NovaException):
                 "than the maximum allowed by flavor %(max_vram)d.")
 
 
+class SecurityProxyNegotiationFailed(NovaException):
+    msg_fmt = _("Failed to negotiate security type with server: %(reason)s")
+
+
+class RFBAuthHandshakeFailed(NovaException):
+    msg_fmt = _("Failed to complete auth handshake: %(reason)s")
+
+
+class RFBAuthNoAvailableScheme(NovaException):
+    msg_fmt = _("No matching auth scheme: allowed types: '%(allowed_types)s', "
+                "desired types: '%(desired_types)s'")
+
+
 class InvalidWatchdogAction(Invalid):
     msg_fmt = _("Provided watchdog action (%(action)s) is not supported.")
 
 
-class NoLiveMigrationForConfigDriveInLibVirt(NovaException):
-    msg_fmt = _("Live migration of instances with config drives is not "
-                "supported in libvirt unless libvirt instance path and "
-                "drive data is shared across compute nodes.")
-
-
 class LiveMigrationWithOldNovaNotSupported(NovaException):
     msg_fmt = _("Live migration with API v2.25 requires all the Mitaka "
                 "upgrade to be complete before it is available.")
 
 
+class SelectionObjectsWithOldRPCVersionNotSupported(NovaException):
+    msg_fmt = _("Requests for Selection objects with alternates are not "
+                "supported in select_destinations() before RPC version 4.5; "
+                "version %(version)s requested.")
+
+
 class LiveMigrationURINotAvailable(NovaException):
     msg_fmt = _('No live migration URI configured and no default available '
                 'for "%(virt_type)s" hypervisor virtualization type.')
@@ -1870,8 +1867,9 @@ class ImageNUMATopologyForbidden(Forbidden):
 
 
 class ImageNUMATopologyAsymmetric(Invalid):
-    msg_fmt = _("Asymmetric NUMA topologies require explicit assignment "
-                "of CPUs and memory to nodes in image or flavor")
+    msg_fmt = _("Instance CPUs and/or memory cannot be evenly distributed "
+                "across instance NUMA nodes. Explicit assignment of CPUs "
+                "and memory to nodes is required")
 
 
 class ImageNUMATopologyCPUOutOfRange(Invalid):
@@ -1944,6 +1942,10 @@ class InvalidToken(Invalid):
     msg_fmt = _("The token '%(token)s' is invalid or has expired")
 
 
+class TokenInUse(Invalid):
+    msg_fmt = _("The generated token is invalid")
+
+
 class InvalidConnectionInfo(Invalid):
     msg_fmt = _("Invalid Connection Info")
 
@@ -2029,14 +2031,6 @@ class MemoryPagesUnsupported(Invalid):
     msg_fmt = _("Host does not support guests with custom memory page sizes")
 
 
-class EnumFieldInvalid(Invalid):
-    msg_fmt = _('%(typename)s in %(fieldname)s is not an instance of Enum')
-
-
-class EnumFieldUnset(Invalid):
-    msg_fmt = _('%(fieldname)s missing field type')
-
-
 class InvalidImageFormat(Invalid):
     msg_fmt = _("Invalid image format '%(format)s'")
 
@@ -2079,6 +2073,11 @@ class LibguestfsCannotReadKernel(Invalid):
     msg_fmt = _("Libguestfs does not have permission to read host kernel.")
 
 
+class MaxDBRetriesExceeded(NovaException):
+    msg_fmt = _("Max retries of DB transaction exceeded attempting to "
+                "perform %(action)s.")
+
+
 class RealtimePolicyNotSupported(Invalid):
     msg_fmt = _("Realtime policy not supported by hypervisor")
 
@@ -2100,7 +2099,11 @@ class BuildRequestNotFound(NotFound):
 
 class AttachInterfaceNotSupported(Invalid):
     msg_fmt = _("Attaching interfaces is not supported for "
-                "instance %(instance)s.")
+                "instance %(instance_uuid)s.")
+
+
+class InstanceDiagnosticsNotSupported(Invalid):
+    msg_fmt = _("Instance diagnostics are not supported by compute node.")
 
 
 class InvalidReservedMemoryPagesOption(Invalid):
@@ -2114,16 +2117,99 @@ class ConcurrentUpdateDetected(NovaException):
                 "Please retry your update")
 
 
+class ResourceClassNotFound(NotFound):
+    msg_fmt = _("No such resource class %(resource_class)s.")
+
+
+class CannotDeleteParentResourceProvider(NovaException):
+    msg_fmt = _("Cannot delete resource provider that is a parent of "
+                "another. Delete child providers first.")
+
+
 class ResourceProviderInUse(NovaException):
     msg_fmt = _("Resource provider has allocations.")
 
 
+class ResourceProviderRetrievalFailed(NovaException):
+    msg_fmt = _("Failed to get resource provider with UUID %(uuid)s")
+
+
+class ResourceProviderAggregateRetrievalFailed(NovaException):
+    msg_fmt = _("Failed to get aggregates for resource provider with UUID"
+                " %(uuid)s")
+
+
+class ResourceProviderTraitRetrievalFailed(NovaException):
+    msg_fmt = _("Failed to get traits for resource provider with UUID"
+                " %(uuid)s")
+
+
+class ResourceProviderCreationFailed(NovaException):
+    msg_fmt = _("Failed to create resource provider %(name)s")
+
+
+class ResourceProviderDeletionFailed(NovaException):
+    msg_fmt = _("Failed to delete resource provider %(uuid)s")
+
+
+class ResourceProviderUpdateFailed(NovaException):
+    msg_fmt = _("Failed to update resource provider via URL %(url)s: "
+                "%(error)s")
+
+
+class PlacementAPIConflict(NovaException):
+    """Any 409 error from placement APIs should use (a subclass of) this
+    exception.
+    """
+    msg_fmt = _("A conflict was encountered attempting to invoke the "
+                "placement API at URL %(url)s: %(error)s")
+
+
+class ResourceProviderUpdateConflict(PlacementAPIConflict):
+    """A 409 caused by generation mismatch from attempting to update an
+    existing provider record or its associated data (aggregates, traits, etc.).
+    """
+    msg_fmt = _("A conflict was encountered attempting to update resource "
+                "provider %(uuid)s (generation %(generation)d): %(error)s")
+
+
+class InventoryWithResourceClassNotFound(NotFound):
+    msg_fmt = _("No inventory of class %(resource_class)s found.")
+
+
+class InvalidResourceClass(Invalid):
+    msg_fmt = _("Resource class '%(resource_class)s' invalid.")
+
+
+class ResourceClassExists(NovaException):
+    msg_fmt = _("Resource class %(resource_class)s already exists.")
+
+
+class ResourceClassInUse(Invalid):
+    msg_fmt = _("Cannot delete resource class %(resource_class)s. "
+                "Class is in use in inventory.")
+
+
+class ResourceClassCannotDeleteStandard(Invalid):
+    msg_fmt = _("Cannot delete standard resource class %(resource_class)s.")
+
+
+class ResourceClassCannotUpdateStandard(Invalid):
+    msg_fmt = _("Cannot update standard resource class %(resource_class)s.")
+
+
+class InvalidResourceAmount(Invalid):
+    msg_fmt = _("Resource amounts must be integers. Received '%(amount)s'.")
+
+
 class InvalidInventory(Invalid):
     msg_fmt = _("Inventory for '%(resource_class)s' on "
                 "resource provider '%(resource_provider)s' invalid.")
 
 
 class InventoryInUse(InvalidInventory):
+    # NOTE(mriedem): This message cannot change without impacting the
+    # nova.scheduler.client.report._RE_INV_IN_USE regex.
     msg_fmt = _("Inventory for '%(resource_classes)s' on "
                 "resource provider '%(resource_provider)s' in use.")
 
@@ -2140,6 +2226,12 @@ class InvalidAllocationCapacityExceeded(InvalidInventory):
                 "amount would exceed the capacity.")
 
 
+class InvalidAllocationConstraintsViolated(InvalidInventory):
+    msg_fmt = _("Unable to create allocation for '%(resource_class)s' on "
+                "resource provider '%(resource_provider)s'. The requested "
+                "amount would violate inventory constraints.")
+
+
 class UnsupportedPointerModelRequested(Invalid):
     msg_fmt = _("Pointer model '%(model)s' requested is not supported by "
                 "host.")
@@ -2158,3 +2250,57 @@ class Unauthorized(NovaException):
 
 class NeutronAdminCredentialConfigurationInvalid(Invalid):
     msg_fmt = _("Networking client is experiencing an unauthorized exception.")
+
+
+class PlacementNotConfigured(NovaException):
+    msg_fmt = _("This compute is not configured to talk to the placement "
+                "service. Configure the [placement] section of nova.conf "
+                "and restart the service.")
+
+
+class InvalidEmulatorThreadsPolicy(Invalid):
+    msg_fmt = _("CPU emulator threads option requested is invalid, "
+                "given: '%(requested)s', available: '%(available)s'.")
+
+
+class BadRequirementEmulatorThreadsPolicy(Invalid):
+    msg_fmt = _("An isolated CPU emulator threads option requires a dedicated "
+                "CPU policy option.")
+
+
+class PowerVMAPIFailed(NovaException):
+    msg_fmt = _("PowerVM API failed to complete for instance=%(inst_name)s.  "
+                "%(reason)s")
+
+
+class TraitNotFound(NotFound):
+    msg_fmt = _("No such trait(s): %(names)s.")
+
+
+class TraitExists(NovaException):
+    msg_fmt = _("The Trait %(name)s already exists")
+
+
+class TraitCannotDeleteStandard(Invalid):
+    msg_fmt = _("Cannot delete standard trait %(name)s.")
+
+
+class TraitInUse(Invalid):
+    msg_fmt = _("The trait %(name)s is in use by a resource provider.")
+
+
+class TraitRetrievalFailed(NovaException):
+    msg_fmt = _("Failed to retrieve traits from the placement API: %(error)s")
+
+
+class TraitCreationFailed(NovaException):
+    msg_fmt = _("Failed to create trait %(name)s: %(error)s")
+
+
+class CannotMigrateWithTargetHost(NovaException):
+    msg_fmt = _("Cannot migrate with target host. Retry without a host "
+                "specified.")
+
+
+class CannotMigrateToSameHost(NovaException):
+    msg_fmt = _("Cannot migrate to the host where the server exists.")
