@@ -296,7 +296,7 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
 
             encryption = encryptors.get_encryption_metadata(context,
                     volume_api, volume_id, connection_info)
-            virt_driver.detach_volume(connection_info, instance, mp,
+            virt_driver.detach_volume(context, connection_info, instance, mp,
                                       encryption=encryption)
         except exception.DiskNotFound as err:
             LOG.warning('Ignoring DiskNotFound exception while '
@@ -304,6 +304,10 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
                         '%(mp)s : %(err)s',
                         {'volume_id': volume_id, 'mp': mp,
                          'err': err}, instance=instance)
+        except exception.DeviceDetachFailed as err:
+            with excutils.save_and_reraise_exception():
+                LOG.warning('Guest refused to detach volume %(vol)s',
+                            {'vol': volume_id}, instance=instance)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception('Failed to detach volume '
@@ -631,7 +635,17 @@ class DriverVolumeBlockDevice(DriverBlockDevice):
         else:
             attachment_ref = volume_api.attachment_get(context,
                                                        self['attachment_id'])
+            # The _volume_attach method stashes a 'multiattach' flag in the
+            # BlockDeviceMapping.connection_info which is not persisted back
+            # in cinder so before we overwrite the BDM.connection_info (via
+            # the update_db decorator on this method), we need to make sure
+            # and preserve the multiattach flag if it's set. Note that this
+            # is safe to do across refreshes because the multiattach capability
+            # of a volume cannot be changed while the volume is in-use.
+            multiattach = self['connection_info'].get('multiattach', False)
             connection_info = attachment_ref['connection_info']
+            if multiattach:
+                connection_info['multiattach'] = True
 
         if 'serial' not in connection_info:
             connection_info['serial'] = self.volume_id
