@@ -19,10 +19,12 @@
 A connection to the VMware vCenter platform.
 """
 
+import os
 import re
 
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import units
 from oslo_utils import versionutils as v_utils
 from oslo_vmware import api
 from oslo_vmware import exceptions as vexc
@@ -30,12 +32,17 @@ from oslo_vmware import pbm
 from oslo_vmware import vim
 from oslo_vmware import vim_util
 
+from nova.compute import power_state
 from nova.compute import task_states
+from nova.compute import utils as compute_utils
 import nova.conf
 from nova import exception
-from nova.i18n import _, _LI, _LE, _LW
+from nova.i18n import _
+from nova.objects import fields as obj_fields
+import nova.privsep.path
 from nova.virt import driver
 from nova.virt.vmwareapi import constants
+from nova.virt.vmwareapi import ds_util
 from nova.virt.vmwareapi import error_util
 from nova.virt.vmwareapi import host
 from nova.virt.vmwareapi import vim_util as nova_vim_util
@@ -48,6 +55,7 @@ LOG = logging.getLogger(__name__)
 CONF = nova.conf.CONF
 
 TIME_BETWEEN_API_CALL_RETRIES = 1.0
+MAX_CONSOLE_BYTES = 100 * units.Ki
 
 
 class VMwareVCDriver(driver.ComputeDriver):
@@ -57,7 +65,8 @@ class VMwareVCDriver(driver.ComputeDriver):
         "has_imagecache": True,
         "supports_recreate": False,
         "supports_migrate_to_same_host": True,
-        "supports_attach_interface": True
+        "supports_attach_interface": True,
+        "supports_multiattach": False
     }
 
     # Legacy nodename is of the form: <mo id>(<cluster name>)
@@ -127,15 +136,22 @@ class VMwareVCDriver(driver.ComputeDriver):
 
     def _check_min_version(self):
         min_version = v_utils.convert_version_to_int(constants.MIN_VC_VERSION)
+        next_min_ver = v_utils.convert_version_to_int(
+            constants.NEXT_MIN_VC_VERSION)
         vc_version = vim_util.get_vc_version(self._session)
-        LOG.info(_LI("VMware vCenter version: %s"), vc_version)
-        if min_version > v_utils.convert_version_to_int(vc_version):
-            # TODO(garyk): enforce this from M
-            LOG.warning(_LW('Running Nova with a VMware vCenter version less '
-                            'than %(version)s is deprecated. The required '
-                            'minimum version of vCenter will be raised to '
-                            '%(version)s in the 13.0.0 release.'),
-                        {'version': constants.MIN_VC_VERSION})
+        LOG.info("VMware vCenter version: %s", vc_version)
+        if v_utils.convert_version_to_int(vc_version) < min_version:
+            raise exception.NovaException(
+                _('Detected vCenter version %(version)s. Nova requires VMware '
+                  'vCenter version %(min_version)s or greater.') % {
+                      'version': vc_version,
+                      'min_version': constants.MIN_VC_VERSION})
+        elif v_utils.convert_version_to_int(vc_version) < next_min_ver:
+            LOG.warning('Running Nova with a VMware vCenter version less '
+                        'than %(version)s is deprecated. The required '
+                        'minimum version of vCenter will be raised to '
+                        '%(version)s in the 16.0.0 release.',
+                        {'version': constants.NEXT_MIN_VC_VERSION})
 
     @property
     def need_legacy_block_device_info(self):
@@ -158,8 +174,7 @@ class VMwareVCDriver(driver.ComputeDriver):
                             CONF.vmware.pbm_default_policy):
                 raise error_util.PbmDefaultPolicyDoesNotExist()
             if CONF.vmware.datastore_regex:
-                LOG.warning(_LW(
-                    "datastore_regex is ignored when PBM is enabled"))
+                LOG.warning("datastore_regex is ignored when PBM is enabled")
                 self._datastore_regex = None
 
     def init_host(self, host):
@@ -172,16 +187,23 @@ class VMwareVCDriver(driver.ComputeDriver):
 
     def _register_openstack_extension(self):
         # Register an 'OpenStack' extension in vCenter
-        LOG.debug('Registering extension %s with vCenter',
-                  constants.EXTENSION_KEY)
         os_extension = self._session._call_method(vim_util, 'find_extension',
                                                   constants.EXTENSION_KEY)
         if os_extension is None:
-            LOG.debug('Extension does not exist. Registering type %s.',
-                      constants.EXTENSION_TYPE_INSTANCE)
-            self._session._call_method(vim_util, 'register_extension',
-                                       constants.EXTENSION_KEY,
-                                       constants.EXTENSION_TYPE_INSTANCE)
+            try:
+                self._session._call_method(vim_util, 'register_extension',
+                                           constants.EXTENSION_KEY,
+                                           constants.EXTENSION_TYPE_INSTANCE)
+                LOG.info('Registered extension %s with vCenter',
+                         constants.EXTENSION_KEY)
+            except vexc.VimFaultException as e:
+                with excutils.save_and_reraise_exception() as ctx:
+                    if 'InvalidArgument' in e.fault_list:
+                        LOG.debug('Extension %s already exists.',
+                                  constants.EXTENSION_KEY)
+                        ctx.reraise = False
+        else:
+            LOG.debug('Extension %s already exists.', constants.EXTENSION_KEY)
 
     def cleanup(self, context, instance, network_info, block_device_info=None,
                 destroy_disks=True, migrate_data=None, destroy_vifs=True):
@@ -194,8 +216,8 @@ class VMwareVCDriver(driver.ComputeDriver):
         # Check if the instance is running already and avoid doing
         # anything if it is.
         state = vm_util.get_vm_state(self._session, instance)
-        ignored_states = ['poweredon', 'suspended']
-        if state.lower() in ignored_states:
+        ignored_states = [power_state.RUNNING, power_state.SUSPENDED]
+        if state in ignored_states:
             return
         # Instance is not up and could be in an unknown state.
         # Be as absolute as possible about getting it back into
@@ -222,7 +244,7 @@ class VMwareVCDriver(driver.ComputeDriver):
         return self._vmops.migrate_disk_and_power_off(context, instance,
                                                       dest, flavor)
 
-    def confirm_migration(self, migration, instance, network_info):
+    def confirm_migration(self, context, migration, instance, network_info):
         """Confirms a resize, destroying the source VM."""
         self._vmops.confirm_migration(migration, instance, network_info)
 
@@ -252,6 +274,21 @@ class VMwareVCDriver(driver.ComputeDriver):
     def get_mks_console(self, context, instance):
         return self._vmops.get_mks_console(instance)
 
+    def get_console_output(self, context, instance):
+        if not CONF.vmware.serial_log_dir:
+            LOG.error("The 'serial_log_dir' config option is not set!")
+            return
+        fname = instance.uuid.replace('-', '')
+        path = os.path.join(CONF.vmware.serial_log_dir, fname)
+        if not os.path.exists(path):
+            LOG.warning('The console log is missing. Check your VSPC '
+                        'configuration', instance=instance)
+            return b""
+        with open(path, 'rb') as fp:
+            read_log_data, remaining = nova.privsep.path.last_bytes(
+                fp, MAX_CONSOLE_BYTES)
+            return read_log_data
+
     def _get_vcenter_uuid(self):
         """Retrieves the vCenter UUID."""
 
@@ -309,8 +346,47 @@ class VMwareVCDriver(driver.ComputeDriver):
         """
         return [self._nodename]
 
+    def get_inventory(self, nodename):
+        """Return a dict, keyed by resource class, of inventory information for
+        the supplied node.
+        """
+        stats = vm_util.get_stats_from_cluster(self._session,
+                                               self._cluster_ref)
+        datastores = ds_util.get_available_datastores(self._session,
+                                                      self._cluster_ref,
+                                                      self._datastore_regex)
+        total_disk_capacity = sum([ds.capacity for ds in datastores])
+        max_free_space = max([ds.freespace for ds in datastores])
+        reserved_disk_gb = compute_utils.convert_mb_to_ceil_gb(
+            CONF.reserved_host_disk_mb)
+        result = {
+            obj_fields.ResourceClass.VCPU: {
+                'total': stats['cpu']['vcpus'],
+                'reserved': CONF.reserved_host_cpus,
+                'min_unit': 1,
+                'max_unit': stats['cpu']['max_vcpus_per_host'],
+                'step_size': 1,
+            },
+            obj_fields.ResourceClass.MEMORY_MB: {
+                'total': stats['mem']['total'],
+                'reserved': CONF.reserved_host_memory_mb,
+                'min_unit': 1,
+                'max_unit': stats['mem']['max_mem_mb_per_host'],
+                'step_size': 1,
+            },
+            obj_fields.ResourceClass.DISK_GB: {
+                'total': total_disk_capacity // units.Gi,
+                'reserved': reserved_disk_gb,
+                'min_unit': 1,
+                'max_unit': max_free_space // units.Gi,
+                'step_size': 1,
+            },
+        }
+        return result
+
     def spawn(self, context, instance, image_meta, injected_files,
-              admin_password, network_info=None, block_device_info=None):
+              admin_password, allocations, network_info=None,
+              block_device_info=None):
         """Create VM instance."""
         self._vmops.spawn(context, instance, image_meta, injected_files,
                           admin_password, network_info, block_device_info)
@@ -320,9 +396,11 @@ class VMwareVCDriver(driver.ComputeDriver):
         """Attach volume storage to VM instance."""
         return self._volumeops.attach_volume(connection_info, instance)
 
-    def detach_volume(self, connection_info, instance, mountpoint,
+    def detach_volume(self, context, connection_info, instance, mountpoint,
                       encryption=None):
         """Detach volume storage to VM instance."""
+        # NOTE(claudiub): if context parameter is to be used in the future,
+        # the _detach_instance_volumes method will have to be updated as well.
         return self._volumeops.detach_volume(connection_info, instance)
 
     def get_volume_connector(self, instance):
@@ -354,22 +432,24 @@ class VMwareVCDriver(driver.ComputeDriver):
             for disk in block_device_mapping:
                 connection_info = disk['connection_info']
                 try:
-                    self.detach_volume(connection_info, instance,
+                    # NOTE(claudiub): Passing None as the context, as it is
+                    # not currently used.
+                    self.detach_volume(None, connection_info, instance,
                                        disk.get('device_name'))
                 except exception.DiskNotFound:
-                    LOG.warning(_LW('The volume %s does not exist!'),
+                    LOG.warning('The volume %s does not exist!',
                                 disk.get('device_name'),
                                 instance=instance)
                 except Exception as e:
                     with excutils.save_and_reraise_exception():
-                        LOG.error(_LE("Failed to detach %(device_name)s. "
-                                      "Exception: %(exc)s"),
+                        LOG.error("Failed to detach %(device_name)s. "
+                                  "Exception: %(exc)s",
                                   {'device_name': disk.get('device_name'),
                                    'exc': e},
                                   instance=instance)
 
     def destroy(self, context, instance, network_info, block_device_info=None,
-                destroy_disks=True, migrate_data=None):
+                destroy_disks=True):
         """Destroy VM instance."""
 
         # Destroy gets triggered when Resource Claim in resource_tracker
@@ -388,8 +468,8 @@ class VMwareVCDriver(driver.ComputeDriver):
             try:
                 self._detach_instance_volumes(instance, block_device_info)
             except vexc.ManagedObjectNotFoundException:
-                LOG.warning(_LW('Instance does not exists. Proceeding to '
-                                'delete instance properties on datastore'),
+                LOG.warning('Instance does not exists. Proceeding to '
+                            'delete instance properties on datastore',
                             instance=instance)
         self._vmops.destroy(instance, destroy_disks)
 
@@ -420,8 +500,7 @@ class VMwareVCDriver(driver.ComputeDriver):
 
     def power_off(self, instance, timeout=0, retry_interval=0):
         """Power off the specified instance."""
-        # TODO(PhilDay): Add support for timeout (clean shutdown)
-        self._vmops.power_off(instance)
+        self._vmops.power_off(instance, timeout, retry_interval)
 
     def power_on(self, context, instance, network_info,
                  block_device_info=None):
@@ -485,13 +564,13 @@ class VMwareVCDriver(driver.ComputeDriver):
         """Efficient override of base instance_exists method."""
         return self._vmops.instance_exists(instance)
 
-    def attach_interface(self, instance, image_meta, vif):
+    def attach_interface(self, context, instance, image_meta, vif):
         """Attach an interface to the instance."""
-        self._vmops.attach_interface(instance, image_meta, vif)
+        self._vmops.attach_interface(context, instance, image_meta, vif)
 
-    def detach_interface(self, instance, vif):
+    def detach_interface(self, context, instance, vif):
         """Detach an interface from the instance."""
-        self._vmops.detach_interface(instance, vif)
+        self._vmops.detach_interface(context, instance, vif)
 
 
 class VMwareAPISession(api.VMwareAPISession):
@@ -505,7 +584,8 @@ class VMwareAPISession(api.VMwareAPISession):
                  retry_count=CONF.vmware.api_retry_count,
                  scheme="https",
                  cacert=CONF.vmware.ca_file,
-                 insecure=CONF.vmware.insecure):
+                 insecure=CONF.vmware.insecure,
+                 pool_size=CONF.vmware.connection_pool_size):
         super(VMwareAPISession, self).__init__(
                 host=host_ip,
                 port=host_port,
@@ -515,9 +595,9 @@ class VMwareAPISession(api.VMwareAPISession):
                 task_poll_interval=CONF.vmware.task_poll_interval,
                 scheme=scheme,
                 create_session=True,
-                wsdl_loc=CONF.vmware.wsdl_location,
                 cacert=cacert,
-                insecure=insecure)
+                insecure=insecure,
+                pool_size=pool_size)
 
     def _is_vim_object(self, module):
         """Check if the module is a VIM Object instance."""
